#!/usr/bin/env python

import Xlib.display
import Xlib.X
import time
import sys
from types import *

keysyms = {
	None: None,
	"Alt_L": 0xFFE9,
	"Shift_L": 0xFFE1,
	"Control_L": 0xFFE3,
	".": 0x02e,
	"+": 0x02b,
	";": 0x03a,
	"\\": 0x05c,
	"Return": 0xFF0d,
	"Space": 0x020,
}
for i in range(ord('A'), ord('Z')+1):
	keysyms[chr(i)] = i

def find_window(top, wm_class, wm_name=None, wm_transient_for=None, child=0, best_child=None):
	for w in top.query_tree().children:
		cls = w.get_wm_class()
		if cls == wm_class:
			transient = wm_transient_for and w.get_wm_transient_for()
			if wm_name and wm_name != w.get_wm_name():
				pass
			elif transient and transient != wm_transient_for:
				pass
			else:
				best_child = w
				if not child:
					return w
		best_child = find_window(w, wm_class, wm_name=wm_name, best_child=best_child, child=child)
		if best_child and not child:
			return best_child
	return best_child


def send_string(win, seq):
	for i in seq:
		if type(i) == type(()):
			(press, release) = i
		else:
			press = release = i
		send_key(win, keysyms[press], keysyms[release])


def send_key(w, press, release):
	if press:
		kc1 = dpy.keysym_to_keycode(press)
	else:
		kc1 = None
	if release:
		kc2 = dpy.keysym_to_keycode(release)
	else:
		kc2 = None

	if press == release and press == None:
		time.sleep(0.2)

	if kc1:
		evt = Xlib.protocol.event.KeyPress(
			time = Xlib.X.CurrentTime,
			root = root,
			window = w,
			same_screen = 1,
			child = 0,
			event_x = 1,
			event_y = 1,
			root_x = 1,
			root_y = 1,
			state = 0,
			detail = kc1
		    )


		w.send_event(evt, Xlib.X.KeyPressMask)

		dpy.sync()

		time.sleep(0.02)

	if kc2:
		evt = Xlib.protocol.event.KeyRelease(
			time = Xlib.X.CurrentTime,
			root = root,
			window = w,
			same_screen = 1,
			child = 0,
			event_x = 1,
			event_y = 1,
			root_x = 1,
			root_y = 1,
			state = 0,
			detail = kc2
		    )

		w.send_event(evt, Xlib.X.KeyReleaseMask)
		dpy.sync()
		time.sleep(0.02)

def focus_window(w):
	evt = Xlib.protocol.event.FocusIn(
		window = w,
		mode = Xlib.X.NotifyNormal,
		detail = Xlib.X.NotifyNonlinear
	    )
	w.send_event(evt, Xlib.X.FocusChangeMask)
	dpy.sync()
	w.set_input_focus(Xlib.X.RevertToParent, Xlib.X.CurrentTime)
	dpy.sync()
	time.sleep(0.02)

time.sleep(0.2)
dpy = Xlib.display.Display()

scr = dpy.screen()
root = scr.root

if __name__ == '__main__':
	### Tell Wordview to print (Ctrl+P) ###
	sequence = [('Control_L', None), 'P', (None, 'Control_L'), 'Return']
	dpy = Xlib.display.Display()

	scr = dpy.screen()
	root = scr.root

	main_win = None
	while not main_win:
		try:
			main_win = find_window(root, ('wine','Wine'), child=0)
		except:
			pass
		time.sleep(1)

	main_parent=main_win.query_tree().parent
	
	# Focus the window
	focus_window(main_win)

	# Induce an error (trying to edit the file), since this sometimes
	# happens anyway
	send_string(main_win, ['A'])
	# And acknowledge it
	dialog = None
	while not dialog:
		dialog = find_window(root, ('wine','Wine'),
				wm_name="Microsoft Word Viewer 97",
				wm_transient_for=main_win,
				child=0)
	send_string(main_win, ['Space'])
	
	# Tell WordView to print
	send_string(main_win, sequence)

	### Acknowledge all dialog boxes until we are killed ###
	while 1:
		try:
			sequence = ['Space']

			dialog = find_window(root, ('wine','Wine'),
				wm_name="Microsoft Word Viewer 97",
				wm_transient_for=main_win,
				child=0)

			if dialog:
				focus_window(dialog)
				send_string(dialog, sequence)
				print "Acknowledged dialog"
			time.sleep(0.5)
		except:
			pass
